/*
 * File : dlStateMachine.h
 *
 * Description : This file contains the declaration of the base class
 *	for all state machine classes - dlStateMachine.
 */

#ifndef _dlStateMachine_h_
#define _dlStateMachine_h_


#include "trace.h"
#include "errorCodes.h"

class dllayer;
class layer;

/*
 * Class : dlStateMachine
 *
 * Description : This class serves as a base class for all 
 *	state machine classes. A member function is provided for
 *	every event that the state machine needs to handle.
 *
 * Attributes:
 *	dllayer *_dl : A pointer to the data-link layer
 */

class dlStateMachine {

 public:
	dlStateMachine(dllayer *dl);
	virtual ~dlStateMachine() {}
	
	// The physical layer indications
	virtual dlStateMachine *PhActivateInd(int &errorCode);
	virtual dlStateMachine *PhDeactivateInd(int &errorCode);

	// The network layer requests and responses
	virtual dlStateMachine *DlConnectReq(int &errorCode); 
	virtual dlStateMachine *DlConnectRes(int &errorCode); 
	virtual dlStateMachine *DlDisconnectReq(int &errorCode); 
	virtual dlStateMachine *DlDataReq(int &errorCode); 
	virtual dlStateMachine *DlResetReq(int &errorCode); 
	virtual dlStateMachine *DlResetRes(int &errorCode); 

	// Frames from the peer data-link layer
	virtual dlStateMachine *HandleIframe(int &errorCode); 
	virtual dlStateMachine *HandleSabm(int &errorCode); 
	virtual dlStateMachine *HandleDisc(int &errorCode); 
	virtual dlStateMachine *HandleDm(int &errorCode); 
	virtual dlStateMachine *HandleUa(int &errorCode); 
	virtual dlStateMachine *HandleRr(int &errorCode); 
	virtual dlStateMachine *HandleRnr(int &errorCode); 
	virtual dlStateMachine *HandleRej(int &errorCode); 
	virtual dlStateMachine *HandleFrmr(int &errorCode); 
	virtual dlStateMachine *HandleUndefined(int &errorCode); 
	virtual dlStateMachine *HandleUnsolFBit(int &errorCode);
	virtual dlStateMachine *HandleIncorrectLength(int &errorCode);
	virtual dlStateMachine *HandleIfrLengthExcess(int &errorCode);

	// Internal events
	virtual dlStateMachine *HandleT1Timer(int &errorCode);
	virtual dlStateMachine *HandleRejTimer(int &errorCode);


	virtual void PrintState();
	protected:
		dllayer *_dl;
};

DECLARE_TRACE_MODULE(dlSmModule);

/*
 * MACRO : CHANGE_STATE
 *
 * Description : Convenience macro to change state.
 *	The argument to the macro is the new state class. 
 *	Creates a new state object, deletes the current
 *  object, returns the new state object.
 */

#define CHANGE_STATE(a) 					\
	dlStateMachine *newState = new a(_dl); 	\
	delete this;							\
	return newState;

#endif
